# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'runner.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_RunDialog(object):
    def setupUi(self, RunDialog):
        if not RunDialog.objectName():
            RunDialog.setObjectName(u"RunDialog")
        RunDialog.resize(753, 645)
        self.gridLayout_2 = QGridLayout(RunDialog)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.splitter = QSplitter(RunDialog)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Vertical)
        self.splitter.setHandleWidth(12)
        self.widget = QWidget(self.splitter)
        self.widget.setObjectName(u"widget")
        self.verticalLayout_2 = QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.scrollArea = QScrollArea(self.widget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 731, 186))
        self.verticalLayout = QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.gridLayout = QGridLayout()
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout.addItem(self.verticalSpacer, 1, 0, 1, 1)

        self.placeholder = QLabel(self.scrollAreaWidgetContents)
        self.placeholder.setObjectName(u"placeholder")
        sizePolicy = QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.placeholder.sizePolicy().hasHeightForWidth())
        self.placeholder.setSizePolicy(sizePolicy)
        self.placeholder.setAlignment(Qt.AlignCenter)

        self.gridLayout.addWidget(self.placeholder, 0, 0, 1, 1)


        self.verticalLayout.addLayout(self.gridLayout)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.verticalLayout_2.addWidget(self.scrollArea)

        self.progressBar = QProgressBar(self.widget)
        self.progressBar.setObjectName(u"progressBar")
        self.progressBar.setValue(0)
        self.progressBar.setTextVisible(True)

        self.verticalLayout_2.addWidget(self.progressBar)

        self.splitter.addWidget(self.widget)
        self.txtOutput = QTextEdit(self.splitter)
        self.txtOutput.setObjectName(u"txtOutput")
        self.txtOutput.setTabChangesFocus(True)
        self.txtOutput.setUndoRedoEnabled(False)
        self.txtOutput.setLineWrapMode(QTextEdit.NoWrap)
        self.txtOutput.setReadOnly(True)
        self.txtOutput.setTabStopWidth(40)
        self.splitter.addWidget(self.txtOutput)

        self.gridLayout_2.addWidget(self.splitter, 0, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.btnOk = QPushButton(RunDialog)
        self.btnOk.setObjectName(u"btnOk")

        self.horizontalLayout.addWidget(self.btnOk)

        self.btnClose = QPushButton(RunDialog)
        self.btnClose.setObjectName(u"btnClose")

        self.horizontalLayout.addWidget(self.btnClose)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer_2)


        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 0, 1, 1)


        self.retranslateUi(RunDialog)

        QMetaObject.connectSlotsByName(RunDialog)
    # setupUi

    def retranslateUi(self, RunDialog):
        RunDialog.setWindowTitle(QCoreApplication.translate("RunDialog", u"Dialog", None))
        self.placeholder.setText(QCoreApplication.translate("RunDialog", u"placeholder", None))
        self.progressBar.setFormat("")
        self.txtOutput.setHtml(QCoreApplication.translate("RunDialog", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Ubuntu'; font-size:11pt; font-weight:400; font-style:normal;\" bgcolor=\"#000000\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>", None))
        self.btnOk.setText(QCoreApplication.translate("RunDialog", u"Run", None))
        self.btnClose.setText(QCoreApplication.translate("RunDialog", u"Close", None))
    # retranslateUi




import os
import sys

from collections import defaultdict
from typing import DefaultDict, List, Optional, Type

from PySide2 import QtWidgets, QtGui, QtCore

import xappt

from xappt_qt.gui.ui.browser import Ui_Browser
from xappt_qt.gui.delegates import SimpleItemDelegate
from xappt_qt.dark_palette import apply_palette
from xappt_qt.constants import *

# noinspection PyUnresolvedReferences
from xappt_qt.gui.resources import icons


class XapptBrowser(QtWidgets.QMainWindow, Ui_Browser):
    ROLE_TOOL_CLASS = QtCore.Qt.UserRole + 1
    ROLE_ITEM_TYPE = QtCore.Qt.UserRole + 2

    ITEM_TYPE_COLLECTION = 0
    ITEM_TYPE_TOOL = 1

    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.setWindowIcon(QtGui.QIcon(":appicon"))
        self.treeTools.setItemDelegate(SimpleItemDelegate())

        self.interfaces = []

        self.populate_plugins()
        self.connect_signals()

        self.txtSearch.setFocus()

    def connect_signals(self):
        self.treeTools.itemActivated.connect(self.item_activated)
        self.treeTools.itemSelectionChanged.connect(self.selection_changed)

        self.txtSearch.textChanged.connect(self.on_filter_tools)
        # noinspection PyAttributeOutsideInit
        self.__txtSearch_keyPressEvent_orig = self.txtSearch.keyPressEvent
        self.txtSearch.keyPressEvent = self._filter_key_press

    def _create_collection_item(self, collection_name: str) -> QtWidgets.QTreeWidgetItem:
        item = QtWidgets.QTreeWidgetItem()
        item.setText(0, collection_name)
        item.setData(0, self.ROLE_TOOL_CLASS, None)
        item.setData(0, self.ROLE_ITEM_TYPE, self.ITEM_TYPE_COLLECTION)
        return item

    def _create_tool_item(self, tool_class: Type[xappt.BaseTool]) -> QtWidgets.QTreeWidgetItem:
        item = QtWidgets.QTreeWidgetItem()
        item.setText(0, tool_class.name())
        item.setToolTip(0, tool_class.help())
        item.setData(0, self.ROLE_TOOL_CLASS, tool_class)
        item.setData(0, self.ROLE_ITEM_TYPE, self.ITEM_TYPE_TOOL)
        return item

    def populate_plugins(self):
        self.treeTools.clear()
        plugin_list: DefaultDict[str, List[Type[xappt.BaseTool]]] = defaultdict(list)

        for _, plugin_class in xappt.plugin_manager.registered_tools():
            collection = plugin_class.collection()
            plugin_list[collection].append(plugin_class)

        for collection in sorted(plugin_list.keys(), key=lambda x: x.lower()):
            collection_item = self._create_collection_item(collection)
            self.treeTools.insertTopLevelItem(self.treeTools.topLevelItemCount(), collection_item)
            for plugin in sorted(plugin_list[collection], key=lambda x: x.name().lower()):
                tool_item = self._create_tool_item(plugin)
                collection_item.addChild(tool_item)
            collection_item.setExpanded(True)

    def item_activated(self, item: QtWidgets.QTreeWidgetItem, column: int):
        item_type = item.data(column, self.ROLE_ITEM_TYPE)
        if item_type != self.ITEM_TYPE_TOOL:
            return
        tool_class = item.data(column, self.ROLE_TOOL_CLASS)
        interface = xappt.get_interface()
        self.interfaces.append(interface)
        interface.invoke(tool_class())

    def selection_changed(self):
        help_text = ""
        selected_items = self.treeTools.selectedItems()
        if len(selected_items):
            tool_class = selected_items[0].data(0, self.ROLE_TOOL_CLASS)  # type: xappt.BaseTool
            if tool_class is not None and len(tool_class.help()):
                help_text = f"{tool_class.name()}: {tool_class.help()}"
        self.labelHelp.setText(help_text)

    def closeEvent(self, event: QtGui.QCloseEvent):
        self.interfaces.clear()

    def _filter_key_press(self, event: QtGui.QKeyEvent):
        if event.key() == QtCore.Qt.Key_Escape:
            self.txtSearch.clear()
        self.__txtSearch_keyPressEvent_orig(event)

    def on_filter_tools(self, text: str):
        if len(text) == 0:
            iterator = QtWidgets.QTreeWidgetItemIterator(self.treeTools, QtWidgets.QTreeWidgetItemIterator.All)
            while iterator.value():
                item = iterator.value()
                item.setHidden(False)
                iterator += 1
            return
        search_terms = [item.lower() for item in text.split(" ") if len(item)]
        self._filter_branch(search_terms, self.treeTools.invisibleRootItem())

    def _filter_branch(self, search_terms: List[str], parent: QtWidgets.QTreeWidgetItem) -> int:
        visible_children = 0
        for c in range(parent.childCount()):
            child = parent.child(c)
            item_type = child.data(0, self.ROLE_ITEM_TYPE)
            if item_type == self.ITEM_TYPE_TOOL:
                child_text = child.text(0).lower()
                child_help = child.toolTip(0).lower()
                visible_children += 1
                item_hidden = False
                for term in search_terms:
                    if term not in child_text and term not in child_help:
                        item_hidden = True
                        break
                child.setHidden(item_hidden)
                if item_hidden:
                    visible_children -= 1
            elif item_type == self.ITEM_TYPE_COLLECTION:
                visible_children += self._filter_branch(search_terms, child)
            else:
                raise NotImplementedError
        parent.setHidden(visible_children == 0)
        return visible_children


def center_window(window: QtWidgets.QMainWindow, screen: Optional[QtGui.QScreen] = None):
    if screen is None:
        app = QtWidgets.QApplication.instance()
        cursor_pos = QtGui.QCursor.pos()
        screen = app.screenAt(cursor_pos)

    screen_rect = screen.availableGeometry()
    window_rect = QtCore.QRect(QtCore.QPoint(0, 0), window.frameSize().boundedTo(screen_rect.size()))
    window.resize(window_rect.size())
    window.move(screen_rect.center() - window_rect.center())


def main(args) -> int:
    app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(args)
    apply_palette(app)

    browser = XapptBrowser()
    center_window(browser)
    browser.show()

    app.setProperty(APP_PROPERTY_RUNNING, True)
    return app.exec_()


def entry_point() -> int:
    os.environ[xappt.INTERFACE_ENV] = APP_INTERFACE_NAME
    return main(sys.argv)


if __name__ == '__main__':
    sys.exit(entry_point())



from typing import Optional

from PySide2 import QtWidgets, QtCore, QtGui

from xappt import BaseTool

from xappt_qt.gui.tool_page import ToolPage
from xappt_qt.gui.ui.runner import Ui_RunDialog

# noinspection PyUnresolvedReferences
from xappt_qt.gui.resources import icons


class RunDialog(QtWidgets.QDialog, Ui_RunDialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self.set_window_attributes()

        self.tool_plugin: Optional[BaseTool] = None
        self.tool_widget: Optional[ToolPage] = None

        self.init_ui()

    def set_window_attributes(self):
        flags = QtCore.Qt.Window
        flags |= QtCore.Qt.WindowCloseButtonHint
        flags |= QtCore.Qt.WindowMinimizeButtonHint
        self.setWindowFlags(flags)
        self.setWindowIcon(QtGui.QIcon(":appicon"))

    def init_ui(self):
        self.placeholder.setVisible(False)

        # noinspection PyArgumentList
        font_size = QtGui.QFontDatabase.systemFont(QtGui.QFontDatabase.GeneralFont).pointSizeF()
        # noinspection PyArgumentList
        mono_font = QtGui.QFontDatabase.systemFont(QtGui.QFontDatabase.FixedFont)
        mono_font.setPointSizeF(font_size)
        self.txtOutput.setFont(mono_font)

        self.splitter.setSizes((self.height(), 0))

    def show_console(self):
        half_height = self.height() * 0.5
        self.splitter.setSizes((half_height, half_height))

    def clear(self):
        if self.tool_widget is not None:
            index = self.gridLayout.indexOf(self.tool_widget)
            self.gridLayout.takeAt(index)
            self.tool_widget.deleteLater()
            self.tool_widget = None
            self.tool_plugin = None
        self.btnOk.setEnabled(True)

    def set_current_tool(self, tool_plugin: BaseTool):
        if self.tool_widget is not None:
            raise RuntimeError("Clear RunDialog before adding a new tool.")
        self.tool_plugin = tool_plugin
        self.tool_widget = ToolPage(self.tool_plugin)
        self.gridLayout.addWidget(self.tool_widget, 0, 0)
        self.setWindowTitle(tool_plugin.name())
        self.tool_widget.setEnabled(True)

    @staticmethod
    def convert_leading_whitespace(s: str, tabwidth: int = 4) -> str:
        leading_spaces = 0
        while True:
            if not len(s):
                break
            if s[0] == " ":
                leading_spaces += 1
            elif s[0] == "\t":
                leading_spaces += tabwidth
            else:
                break
            s = s[1:]
        return f"{'&nbsp;' * leading_spaces}{s}"

    def add_output_line(self, s: str, error: bool = False):
        s = self.convert_leading_whitespace(s)
        self.txtOutput.moveCursor(QtGui.QTextCursor.End)
        if error:
            self.txtOutput.insertHtml(f'<span style="color: #f55">{s}</span><br />\n')
        else:
            self.txtOutput.insertHtml(f'<span style="color: #ccc">{s}</span><br />\n')
        self.txtOutput.moveCursor(QtGui.QTextCursor.End)
        max_scroll = self.txtOutput.verticalScrollBar().maximum()
        self.txtOutput.verticalScrollBar().setValue(max_scroll)
        # noinspection PyArgumentList
        QtWidgets.QApplication.instance().processEvents()

    def add_error_line(self, s: str):
        self.add_output_line(s, True)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

# adapted from https://gist.github.com/lschmierer/443b8e21ad93e2a2d7eb

from PySide2.QtGui import QPalette, QColor
from PySide2.QtCore import Qt

PRIMARY_COLOR = QColor(32, 32, 32)
SECONDARY_COLOR = PRIMARY_COLOR.darker(150)
HIGHLIGHT_COLOR = QColor(128, 176, 73)
BRIGHT_COLOR = QColor(192, 192, 192)
DISABLED_COLOR = QColor(64, 64, 64)


# noinspection PyUnresolvedReferences
def apply_palette(app):
    app.setStyle("Fusion")

    dark_palette = QPalette()

    dark_palette.setColor(QPalette.Window, PRIMARY_COLOR)
    dark_palette.setColor(QPalette.WindowText, BRIGHT_COLOR)
    dark_palette.setColor(QPalette.Base, SECONDARY_COLOR)
    dark_palette.setColor(QPalette.AlternateBase, PRIMARY_COLOR)
    dark_palette.setColor(QPalette.ToolTipBase, BRIGHT_COLOR)
    dark_palette.setColor(QPalette.ToolTipText, BRIGHT_COLOR)
    dark_palette.setColor(QPalette.Text, BRIGHT_COLOR)
    dark_palette.setColor(QPalette.Button, PRIMARY_COLOR)
    dark_palette.setColor(QPalette.ButtonText, BRIGHT_COLOR)
    dark_palette.setColor(QPalette.BrightText, Qt.red)
    dark_palette.setColor(QPalette.Link, HIGHLIGHT_COLOR)
    dark_palette.setColor(QPalette.Highlight, HIGHLIGHT_COLOR)
    dark_palette.setColor(QPalette.HighlightedText, SECONDARY_COLOR)

    dark_palette.setColor(QPalette.Light, PRIMARY_COLOR)

    dark_palette.setColor(QPalette.Disabled,
                          QPalette.WindowText, DISABLED_COLOR)
    dark_palette.setColor(QPalette.Disabled,
                          QPalette.Text, DISABLED_COLOR)
    dark_palette.setColor(QPalette.Disabled,
                          QPalette.ButtonText, DISABLED_COLOR)

    app.setPalette(dark_palette)
    app.setStyleSheet("""
        QToolTip {{ 
            color: {2}; 
            background-color: {1}; 
            border: 1px solid {0};
            padding: 2px 8px;
        }}
    """.format(HIGHLIGHT_COLOR.name(),
               PRIMARY_COLOR.name(),
               BRIGHT_COLOR.name()))



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'browser.ui'
##
## Created by: Qt User Interface Compiler version 5.15.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *


class Ui_Browser(object):
    def setupUi(self, Browser):
        if not Browser.objectName():
            Browser.setObjectName(u"Browser")
        Browser.resize(553, 709)
        self.centralwidget = QWidget(Browser)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.labelHelp = QLabel(self.centralwidget)
        self.labelHelp.setObjectName(u"labelHelp")
        self.labelHelp.setWordWrap(True)

        self.gridLayout.addWidget(self.labelHelp, 3, 0, 1, 1)

        self.treeTools = QTreeWidget(self.centralwidget)
        __qtreewidgetitem = QTreeWidgetItem()
        __qtreewidgetitem.setText(0, u"1");
        self.treeTools.setHeaderItem(__qtreewidgetitem)
        self.treeTools.setObjectName(u"treeTools")
        self.treeTools.setAlternatingRowColors(True)
        self.treeTools.setRootIsDecorated(True)
        self.treeTools.setSortingEnabled(False)
        self.treeTools.header().setVisible(False)

        self.gridLayout.addWidget(self.treeTools, 1, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.txtSearch = QLineEdit(self.centralwidget)
        self.txtSearch.setObjectName(u"txtSearch")

        self.horizontalLayout.addWidget(self.txtSearch)

        self.btnClear = QToolButton(self.centralwidget)
        self.btnClear.setObjectName(u"btnClear")

        self.horizontalLayout.addWidget(self.btnClear)


        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 1)

        Browser.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(Browser)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 553, 27))
        Browser.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(Browser)
        self.statusbar.setObjectName(u"statusbar")
        Browser.setStatusBar(self.statusbar)
        QWidget.setTabOrder(self.txtSearch, self.btnClear)
        QWidget.setTabOrder(self.btnClear, self.treeTools)

        self.retranslateUi(Browser)
        self.btnClear.clicked.connect(self.txtSearch.clear)

        QMetaObject.connectSlotsByName(Browser)
    # setupUi

    def retranslateUi(self, Browser):
        Browser.setWindowTitle(QCoreApplication.translate("Browser", u"Xappt Browser", None))
        self.labelHelp.setText("")
        self.txtSearch.setPlaceholderText(QCoreApplication.translate("Browser", u"Search", None))
        self.btnClear.setText(QCoreApplication.translate("Browser", u"\u232b", None))
    # retranslateUi




import os
import sys

from PySide2 import QtWidgets

import xappt
from xappt import BaseTool

from xappt_qt.dark_palette import apply_palette

from xappt_qt.gui.run_dialog import RunDialog

from xappt_qt.constants import *

# noinspection PyUnresolvedReferences
from xappt_qt.gui.resources import icons

os.environ["QT_STYLE_OVERRIDE"] = "Fusion"
os.environ[xappt.INTERFACE_ENV] = APP_INTERFACE_NAME


@xappt.register_plugin
class QtInterface(xappt.BaseInterface):
    class __QtInterfaceInner:
        def __init__(self):
            self.app = QtWidgets.QApplication.instance() or QtWidgets.QApplication(sys.argv)
            apply_palette(self.app)

        def exec_(self):
            if self.app.property(APP_PROPERTY_RUNNING):
                return
            self.app.setProperty(APP_PROPERTY_RUNNING, True)
            self.app.exec_()

        def exit(self, return_code=0):
            self.app.exit(return_code)

    instance = None

    def __new__(cls):
        if not QtInterface.instance:
            QtInterface.instance = QtInterface.__QtInterfaceInner()
        return super().__new__(cls)

    def __init__(self):
        super().__init__()
        self.runner = RunDialog()
        self.runner.btnOk.clicked.connect(self.on_run)
        self.runner.btnClose.clicked.connect(self.on_close)

    @classmethod
    def name(cls) -> str:
        return APP_INTERFACE_NAME

    def invoke(self, plugin: BaseTool, **kwargs):
        self.runner.clear()
        self.runner.set_current_tool(plugin)
        self.runner.show()
        self.instance.exec_()

    def message(self, message: str):
        xappt.log.info(message)
        QtWidgets.QMessageBox.information(self.runner, APP_TITLE, message)

    def warning(self, message: str):
        xappt.log.warning(message)
        QtWidgets.QMessageBox.warning(self.runner, APP_TITLE, message)

    def error(self, message: str):
        xappt.log.error(message)
        QtWidgets.QMessageBox.critical(self.runner, APP_TITLE, message)

    def ask(self, message: str) -> bool:
        buttons = QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No
        ask_result = QtWidgets.QMessageBox.question(self.runner, APP_TITLE, message, buttons=buttons,
                                                    defaultButton=QtWidgets.QMessageBox.No)
        return ask_result == QtWidgets.QMessageBox.Yes

    def progress_start(self):
        self.runner.progressBar.setRange(0, 100)
        self.instance.app.processEvents()

    def progress_update(self, message: str, percent_complete: float):
        self.runner.progressBar.setValue(100.0 * percent_complete)
        self.runner.progressBar.setFormat(message)
        self.instance.app.processEvents()

    def progress_end(self):
        self.runner.progressBar.setValue(0)
        self.runner.progressBar.setFormat("")
        self.instance.app.processEvents()

    def on_run(self):
        try:
            self.runner.tool_plugin.validate()
        except xappt.ParameterValidationError as e:
            self.message(str(e))
            return
        self.runner.btnOk.setEnabled(False)
        self.runner.tool_widget.setEnabled(False)
        result = self.runner.tool_plugin.execute(interface=self)
        if result != 0:
            self.runner.btnOk.setEnabled(True)
            self.runner.tool_widget.setEnabled(True)

    def on_close(self):
        self.runner.close()



from typing import Any, Callable, Dict, List, Optional, Type

from PySide2 import QtWidgets, QtCore

import xappt

from xappt_qt.gui.widgets import *
from xappt_qt.gui.delegates import SimpleItemDelegate


class ToolPage(QtWidgets.QWidget):
    def __init__(self, tool: xappt.BaseTool, parent: Optional[QtWidgets.QWidget] = None):
        super().__init__(parent=parent)

        self.convert_dispatch: Dict[Type, Callable] = {
            int: self._convert_int,
            bool: self._convert_bool,
            float: self._convert_float,
            str: self._convert_str,
            list: self._convert_list,
        }

        self.tool = tool
        self.build_ui()

    # noinspection PyAttributeOutsideInit
    def build_ui(self):
        self.grid = QtWidgets.QGridLayout()
        self.grid.setColumnStretch(0, 0)
        self.grid.setColumnStretch(1, 1)
        self.grid.setHorizontalSpacing(16)
        self.grid.setVerticalSpacing(8)

        self.setLayout(self.grid)
        self._load_tool_parameters()

    @staticmethod
    def get_caption(param: xappt.Parameter) -> str:
        caption_default = param.name.replace("_", " ").title()
        caption = param.options.get("caption", caption_default)
        return caption

    def update_tool_choices(self, param: xappt.Parameter):
        """ Given that multiple parameter types can be updated at runtime,
        it's easier just to remove and recreate the widget rather than
        reimplementing a lot of the same functionality to update existing
        widgets. """
        widget = param.metadata.get('widget')
        if widget is None:
            return

        # find and remove existing widget
        index = self.grid.indexOf(widget)
        row, column, *_ = self.grid.getItemPosition(index)
        self.grid.takeAt(index)
        widget.deleteLater()
        param.on_choices_changed.clear()

        # create a new widget to replace it
        new_widget = self.convert_parameter(param)
        self.grid.addWidget(new_widget, row, column)
        param.metadata['widget'] = new_widget
        param.on_choices_changed.add(self.update_tool_choices)

    def _load_tool_parameters(self):
        for i, param in enumerate(self.tool.parameters()):
            label = QtWidgets.QLabel(self.get_caption(param))
            label.setAlignment(QtCore.Qt.AlignRight | QtCore.Qt.AlignVCenter)
            label.setToolTip(param.description)
            widget = self.convert_parameter(param)
            widget.setToolTip(param.description)
            self.grid.addWidget(label, i, 0)
            self.grid.addWidget(widget, i, 1)
            param.metadata['label'] = label
            param.metadata['widget'] = widget  # this lets us avoid lambdas
            param.on_choices_changed.add(self.update_tool_choices)
            param.on_value_changed.add(self.widget_value_updated)
            param.on_options_changed.add(self.widget_options_updated)

    def convert_parameter(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        convert_fn = self.convert_dispatch.get(param.data_type)
        if convert_fn is not None:
            return convert_fn(param)
        return QtWidgets.QWidget()

    def _convert_int(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        if param.choices is not None:
            return self._convert_int_choice(param)
        else:
            return self._convert_int_spin(param)

    def _convert_int_choice(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        w = QtWidgets.QComboBox()
        w.addItems(param.choices)
        for v in (param.value, param.default):
            if v is not None:
                if isinstance(v, str):
                    if v in param.choices:
                        index = w.findText(v)
                        w.setCurrentIndex(index)
                elif isinstance(v, int):
                    if 0 <= v < w.count():
                        w.setCurrentIndex(v)
                break
        else:
            param.value = w.currentIndex()
        w.currentIndexChanged[int].connect(lambda x: self.update_tool_param(param.name, x))
        param.metadata['ui-setter'] = w.setCurrentIndex
        return w

    def _convert_int_spin(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        w = QtWidgets.QSpinBox(parent=self)
        minimum = param.options.get("minimum", -999999999)
        maximum = param.options.get("maximum", 999999999)
        w.setMinimum(minimum)
        w.setMaximum(maximum)
        for v in (param.value, param.default):
            if v is not None:
                w.setValue(v)
                break
        else:
            param.value = w.value()
        w.valueChanged[int].connect(lambda x: self.update_tool_param(param.name, x))
        param.metadata['ui-setter'] = w.setValue
        return w

    def _convert_bool(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        w = QtWidgets.QCheckBox()
        for v in (param.value, param.default):
            if v is not None:
                w.setChecked(v)
                break
        else:
            param.value = w.isChecked()
        w.stateChanged.connect(lambda x: self.update_tool_param(param.name, x == QtCore.Qt.Checked))
        param.metadata['ui-setter'] = w.setChecked
        return w

    def _convert_str(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        if param.choices is not None:
            return self._convert_str_choice(param)
        else:
            return self._convert_str_edit(param)

    def _convert_str_choice(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        w = QtWidgets.QComboBox()
        w.addItems(param.choices)
        for v in (param.value, param.default):
            if v is not None:
                if v in param.choices:
                    index = w.findText(param.default)
                    w.setCurrentIndex(index)
                    break
        else:
            param.value = w.currentText()
        w.currentIndexChanged[str].connect(lambda x: self.update_tool_param(param.name, x))
        param.metadata['ui-setter'] = lambda s, widget=w: widget.setCurrentIndex(widget.findText(s))
        return w

    def _convert_str_edit(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        ui = param.options.get("ui")
        if ui == "folder-select":
            w = FileEdit(mode=FileEdit.MODE_CHOOSE_DIR)
            w.onSetFile.connect(lambda x: self.update_tool_param(param.name, x))
        elif ui == "file-open":
            w = FileEdit(accept=param.options.get("accept"), mode=FileEdit.MODE_OPEN_FILE)
            w.onSetFile.connect(lambda x: self.update_tool_param(param.name, x))
        elif ui == "file-save":
            w = FileEdit(accept=param.options.get("accept"), mode=FileEdit.MODE_SAVE_FILE)
            w.onSetFile.connect(lambda x: self.update_tool_param(param.name, x))
        else:
            w = QtWidgets.QLineEdit()
            w.textChanged.connect(lambda x: self.update_tool_param(param.name, x))

        for v in (param.value, param.default):
            if v is not None:
                w.setText(param.value)
                break
        else:
            w.setText("")
        param.metadata['ui-setter'] = w.setText
        return w

    # noinspection DuplicatedCode
    def _convert_float(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        w = QtWidgets.QDoubleSpinBox(parent=self)
        minimum = param.options.get("minimum", -999999999.0)
        maximum = param.options.get("maximum", 999999999.0)
        w.setMinimum(minimum)
        w.setMaximum(maximum)
        if param.default is not None:
            w.setValue(param.default)
        param.value = w.value()
        w.valueChanged[float].connect(lambda x: self.update_tool_param(param.name, x))
        param.metadata['ui-setter'] = w.setValue
        return w

    def _convert_list(self, param: xappt.Parameter) -> QtWidgets.QWidget:
        w = QtWidgets.QListWidget()
        w.setItemDelegate(SimpleItemDelegate())
        w.setSelectionMode(QtWidgets.QAbstractItemView.SelectionMode.ExtendedSelection)
        w.setAlternatingRowColors(True)
        w.setSpacing(2)
        if param.choices is not None:
            w.addItems(param.choices)
        for v in (param.value, param.default):
            if v is not None:
                for item in v:
                    for match in w.findItems(item, QtCore.Qt.MatchExactly):
                        match.setSelected(True)
                break
        param.value = self._get_selected_list_items(w)
        w.itemSelectionChanged.connect(lambda: self.update_list_param(param.name))
        param.metadata['ui-setter'] = lambda value, widget=w: self.set_list_value(value, widget)
        return w

    @staticmethod
    def _get_selected_list_items(widget: QtWidgets.QListWidget) -> List[str]:
        return [item.text() for item in widget.selectedItems()]

    def update_list_param(self, name: str):
        param: xappt.Parameter = getattr(self.tool, name)
        widget: QtWidgets.QListWidget = param.metadata.get('widget')
        if widget is None:
            return
        selected_items = self._get_selected_list_items(widget)
        param.on_value_changed.paused = True
        param.value = param.validate(selected_items)
        param.on_value_changed.paused = False

    def update_tool_param(self, name: str, value: Any):
        param: xappt.Parameter = getattr(self.tool, name)
        param.value = param.validate(value)

    @staticmethod
    def set_list_value(items: List[str], widget: QtWidgets.QListWidget):
        for item in items:
            for match in widget.findItems(item, QtCore.Qt.MatchExactly):
                match.setSelected(True)

    @staticmethod
    def widget_value_updated(param: xappt.Parameter):
        setter = param.metadata.get('ui-setter')
        if setter is None:
            return
        setter(param.value)

    @staticmethod
    def widget_options_updated(param: xappt.Parameter):
        label: Optional[QtWidgets.QLabel] = param.metadata.get('label')
        widget: Optional[QtWidgets.QWidget] = param.metadata.get('widget')

        visible = param.option("visible", True)
        if label is not None:
            label.setVisible(visible)
        if widget is not None:
            widget.setVisible(visible)

        enabled = param.option("enabled", True)
        if widget is not None:
            widget.setEnabled(enabled)
